package be.iminds.ilabt.jfed.experimenter_gui.dialogs;

import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Testbed;
import be.iminds.ilabt.jfed.lowlevel.GeniUserProvider;
import be.iminds.ilabt.jfed.lowlevel.connection.*;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.util.JFedTrustStore;
import be.iminds.ilabt.jfed.util.RFC3339Util;
import be.iminds.ilabt.jfed.util.TimeUtil;
import com.sun.javafx.webkit.WebConsoleListener;
import javafx.application.Platform;
import javafx.beans.value.ObservableValue;
import javafx.concurrent.Worker;
import javafx.fxml.FXML;
import javafx.scene.control.Label;
import javafx.stage.Window;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.inject.Inject;
import java.text.ParseException;
import java.time.Duration;
import java.time.Instant;

public class GDPRDialog extends JFedBrowser {
    private static final Logger LOG = LoggerFactory.getLogger(GDPRDialog.class);

    private Testbed testbed;

    @FXML protected Label untilLabel;

    protected Boolean tcApproved = null;
    protected Boolean tcDeclined = null;
    protected Instant approvedUntil = null;

    public static class GDPRDialogJavaScriptBridge extends JavaScriptBridge {
        @Nonnull private final GDPRDialog gdprDialog;
        public GDPRDialogJavaScriptBridge(@Nonnull GDPRDialog gdprDialog) {
            super(gdprDialog);
            this.gdprDialog = gdprDialog;
        }

        public void approve() {
            LOG.debug("GDPRDialogJavaScriptBridge.approve");
//            gdprDialog.centerBox.setStyle("-fx-background-color: lawngreen");
            gdprDialog.scene.getRoot().setStyle("-fx-background-color: lawngreen");
            gdprDialog.tcApproved = true;
            gdprDialog.approvedUntil = null;
            gdprDialog.tcDeclined = null;
            gdprDialog.checkUntil();
            gdprDialog.updateUntilView();
        }

        /**
         * Use javascript Date.getTime()  to find this
         */
        public void approveWithDateInMillisecondsSinceEpoch(long dateInMsSinceEpoch) {
            LOG.debug("GDPRDialogJavaScriptBridge.approveWithDateInMillisecondsSinceEpoch("+dateInMsSinceEpoch+")");
            Instant instant = Instant.ofEpochMilli(dateInMsSinceEpoch);
            gdprDialog.scene.getRoot().setStyle("-fx-background-color: lawngreen");
            gdprDialog.tcApproved = true;
            gdprDialog.approvedUntil = instant;
            gdprDialog.tcDeclined = null;
            gdprDialog.checkUntil();
            gdprDialog.updateUntilView();
        }

        public void approveDaysFromNow(int daysFromNow) {
            LOG.debug("GDPRDialogJavaScriptBridge.approveDaysFromNow("+daysFromNow+")");
            Instant instant = Instant.now().plus(Duration.ofDays(daysFromNow));
            gdprDialog.scene.getRoot().setStyle("-fx-background-color: lawngreen");
            gdprDialog.tcApproved = true;
            gdprDialog.approvedUntil = instant;
            gdprDialog.tcDeclined = null;
            gdprDialog.checkUntil();
            gdprDialog.updateUntilView();
        }

        /**
         * Note that RFC3339 should also be parsable by this
         * @param dateString
         */
        public void approveWithDateISO8601(@Nullable String dateString) {
            LOG.debug("GDPRDialogJavaScriptBridge.approveWithDateISO8601(\""+dateString+"\")");
            gdprDialog.scene.getRoot().setStyle("-fx-background-color: lawngreen");
            gdprDialog.tcApproved = true;
            try {
                gdprDialog.approvedUntil = dateString == null || dateString.trim().isEmpty()
                        || dateString.trim().equals("null") || dateString.trim().equals("undefined") ?
                        null :
                        RFC3339Util.iso8601StringToDate(dateString).toInstant();
            } catch (ParseException e) {
                LOG.debug("GDPRDialogJavaScriptBridge.approve date parse error", e);
            }
            gdprDialog.tcDeclined = null;
            gdprDialog.checkUntil();
            gdprDialog.updateUntilView();
        }

        public void decline() {
            LOG.debug("GDPRDialogJavaScriptBridge.decline");
//            gdprDialog.centerBox.setStyle("-fx-background-color: orangered");
            gdprDialog.scene.getRoot().setStyle("-fx-background-color: orangered");
            gdprDialog.tcApproved = null;
            gdprDialog.approvedUntil = null;
            gdprDialog.tcDeclined = true;
            gdprDialog.checkUntil();
            gdprDialog.updateUntilView();
        }
        public void close() {
            LOG.debug("GDPRDialogJavaScriptBridge.close");
            Window stage = gdprDialog.scene.getWindow();
            stage.hide();
        }
    }

    @Inject
    public GDPRDialog(@Nonnull GeniUserProvider geniUserProvider,
                      @Nonnull SfaConnectionPool connectionPool,
                      @Nonnull JFedTrustStore trustStore,
                      @Nonnull JFedPreferences preferences) {
        super(geniUserProvider, connectionPool, trustStore, preferences);
    }

    private void checkUntil() {
        if (approvedUntil != null && approvedUntil.isBefore(Instant.now())) {
            LOG.error("Approval is in the past");
            tcApproved = false;
        }
    }

    private void updateUntilView() {
        untilLabel.setVisible(approvedUntil != null);
        if (approvedUntil != null) {
            untilLabel.setText("Approved until "+RFC3339Util.instantToRFC3339String(approvedUntil)+" ("+
                    TimeUtil.calcIn(approvedUntil) +")");
        }
    }

    @Nullable
    public Boolean hasApproved() {
        return tcApproved;
    }

    @Nullable
    public Boolean hasDeclined() {
        return tcDeclined;
    }

    @Nullable
    public Instant getApprovedUntil() {
        return approvedUntil;
    }

    public void setTestbed(Testbed testbed) {
        this.testbed = testbed;

        loadPageWhenReady();
    }

    @FXML
    @Override
    protected void initialize() {
        webEngine = webView.getEngine();

        URLStreamFactoryCustomizer.useDedicatedProxyForWebkit(
                new MyHttpUrlStreamHandler(), "https");
//        webEngine.load("https://www.wilab2.ilabt.iminds.be:12369/reservation/sfareservation.php3");
//        webEngine.load("https://jfed.ilabt.imec.be");
        initialized = true;

        webEngine.getLoadWorker().stateProperty().addListener(
                (ObservableValue<? extends Worker.State> ov, Worker.State oldState,
                 Worker.State newState) -> {
                    if (newState == Worker.State.SUCCEEDED) {
                        jsBridgeBackoff = 1;
                        registerJsBridge();
                    }
                });

        WebConsoleListener.setDefaultListener((webView, message, lineNumber, sourceId) -> {
            LOG.debug("[Webengine message] "+message + "[at " + lineNumber + "]");
        });

        loadPageWhenReady();
    }

    @Override
    protected void loadPageWhenReady() {
        if (!initialized)
            return;

        if (this.testbed == null)
            return;

        Platform.runLater(this::registerJsBridge);
        loadPage();
    }

    @Override
    protected Object makeJavascriptBridge() {
        return new GDPRDialogJavaScriptBridge(this);
    }

    protected void loadPage() {
        if (this.testbed.getGdprInfo() == null) {
            LOG.warn("Cannot show Terms & Conditions page for Testbed without gdprInfo");
            return;
        }

        String gdprUrl = this.testbed.getGdprInfo().getGrpdUrl();
        if (gdprUrl == null) {
            LOG.warn("Cannot show Terms & Conditions page for Testbed without gdprInfo.grpdUrl");
            return;
        }

        headerLabel.setText("Terms & Conditions for "+testbed.getLongName());

        //gdprUrl = "https://www.wall2.ilabt.iminds.be:12369/gdpr/login.php";
        webEngine.load(gdprUrl);
    }

}
